﻿--[[
internals of all customButtons:

XUB_ConfigButtons[1]: name of the button
XUB_ConfigButtons[2]: typ (spell, spellblink, buff, buffblink, debuff, debuffblink, health, healthblink, use, always or macro)
XUB_ConfigButtons[3]: spellname / macroIcon / useItem
XUB_ConfigButtons[4]: spellrank / macroText
XUB_ConfigButtons[5]: debuff: type/buff: spellname / health: threshold
]]

function XUB_VerifyInputs()

	if ( XUB_TestButtons(XUB_ConfigPanel1ButtonsEdit:GetText()) == 0 ) and ( XUB_TestGroups() == 0 ) and ( XUB_TestProfiles() == 0 ) then
		XUB_ConfigSaveAndExit:Show();
        XUB_ConfigGetMacro:Show();
	else
		XUB_ConfigSaveAndExit:Hide();
        XUB_ConfigGetMacro:Hide();
	end

end

--
-- This method generates a set of button definitions based on the spell book.  For spells
-- that have one or more Ranks, the system generates "bonus buttons" which is the name of the
-- spell without a Rank.  In this case, it allows the mod user to create a button in the game
-- that when pressed will cause the highest possible rank available for the selected unit
-- to be cast on it.  This is a new WOW 2.0 feature.  XUB_DefaultButtons
--
function XUB_GenerateDefaultSpells()

	XUB_GenerateDefaultSpellsAsList()

end

function XUB_GenerateDefaultSpellsAsList()

 	local i=1
 	local myText = "";
 	local myBonusText = "\n";
 	local SpellName,l_Rank = GetSpellName(i,SpellBookFrame.bookType );
 	local myLastSpell = "**";

 	XUB_DefaultButtons = {};
    XUB_DefaultButtonsArray = {};

 	while ( SpellName ~= nil ) do

        local mySpellVariable

		if ( SpellName ~= myLastSpell ) then
        --if we have a new spellname then let's add a version without rank
			if ( l_Rank:find(RANK) ~= nil ) then
				mySpellVariable = string.gsub(SpellName, " ", "");
				mySpellVariable = string.gsub(mySpellVariable , ":","" );
				mySpellVariable = string.gsub(mySpellVariable , "'","" );
				mySpellVariable = string.gsub(mySpellVariable , "%)","" );
				mySpellVariable = string.gsub(mySpellVariable , "%(","" );

		 		XUB_DefaultButtons[#XUB_DefaultButtons+1] = mySpellVariable .. ",spell," .. SpellName .. ","
                XUB_DefaultButtonsArray[#XUB_DefaultButtonsArray+1] = { strsplit(",",XUB_DefaultButtons[#XUB_DefaultButtons]) };
		 	end
		end

		mySpellVariable = string.gsub(SpellName .. l_Rank, " ", "");
		mySpellVariable = string.gsub(mySpellVariable , ":","" );
		mySpellVariable = string.gsub(mySpellVariable , "'","" );
		mySpellVariable = string.gsub(mySpellVariable , "%)","" );
		mySpellVariable = string.gsub(mySpellVariable , "%(","" );

 		XUB_DefaultButtons[#XUB_DefaultButtons+1] = mySpellVariable .. ",spell," .. SpellName .. "," .. l_Rank
        XUB_DefaultButtonsArray[#XUB_DefaultButtonsArray+1] = { strsplit(",",XUB_DefaultButtons[#XUB_DefaultButtons]) };
 		myLastSpell = SpellName;

 		i=i+1
 		SpellName,l_Rank = GetSpellName(i,SpellBookFrame.bookType );

 	end

end

function XUB_LoadPreviousSpells()

	local myLoadText = "";
    local myPrefix = "";
	local i = 0;

	-- Load the custom buttons
	if ( XUB_ConfigButtons ~= nil ) then
		for i = 1,#XUB_ConfigButtons do
            myPrefix = "";
            for e = 1,#XUB_ConfigButtons[i] do
                if ( XUB_ConfigButtons[i][e] ~= nil ) then
--                    if ( XUB_ConfigButtons[i][2] ~= "spell" ) then
                        myLoadText = myLoadText .. myPrefix .. XUB_ConfigButtons[i][e];
                        myPrefix = ",";
--                    end
                end
            end
            myLoadText = myLoadText .. "\n"
		end

		XUB_ConfigPanel1ButtonsEdit:SetText( myLoadText );
	end


	--
	-- Groups
	--
	if ( XUB_ConfigGroups ~= nil ) then
		myLoadText = "";
        myLastGroup = "";
		for i = 1,#XUB_ConfigGroups do
            myPrefix = "";
            if ( XUB_ConfigGroups[i][1] ~= myLastGroup ) then
                if ( myLastGroup ~= "" ) then
                    myLoadText = myLoadText .. "\n";
                end
                myLastGroup = XUB_ConfigGroups[i][1];
            end

            for e = 1,#XUB_ConfigGroups[i] do
                if ( XUB_ConfigGroups[i][e] ~= nil ) then
                    myLoadText = myLoadText .. myPrefix .. XUB_ConfigGroups[i][e];
                    myPrefix = ",";
                end
            end
            myLoadText = myLoadText .. "\n"
		end

		XUB_ConfigPanel2GroupsEdit:SetText( myLoadText );
	end

	--
	-- Profiles
	--
	if ( XUB_ConfigProfiles ~= nil ) then
		myLoadText = "";
        myLastGroup = "";
		for i = 1,#XUB_ConfigProfiles do
            myPrefix = "";
            myMerge = XUB_ConfigProfiles[i][1] .. "~" .. XUB_ConfigProfiles[i][2];
            if ( myMerge ~= myLastGroup ) then
                if ( myLastGroup ~= "" ) then
                    myLoadText = myLoadText .. "\n";
                end
                myLastGroup = myMerge;
            end

            for e = 1,#XUB_ConfigProfiles[i] do
                if ( XUB_ConfigProfiles[i][e] ~= nil ) then
                    myLoadText = myLoadText .. myPrefix .. XUB_ConfigProfiles[i][e];
                    myPrefix = ",";
                end
            end
            myLoadText = myLoadText .. "\n"
		end

		XUB_ConfigPanel3ProfilesEdit:SetText( myLoadText );
	end

end

--
-- Tests the syntax of the Button definition
--
function XUB_TestButtons(aText)

	local myError = 0;

	local myButtons = XUB_SplitTextIntoTable( aText );

	local COMMAND=2;

	for i = 1,# myButtons do
		local myTrim,myItems = XUB_SplitCommaAndTrim(myButtons[i]);

		if ( myTrim[COMMAND] == "use" ) then

			if ( #myTrim ~= 3 ) and ( #myTrim ~= 4 ) then
				ShowMessage( "USE specification requires 3 or 4 elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end
			end
--		elseif ( #myTrim <= 3 ) then
--			ShowMessage( "BUTTON has too few items"  .. CCC( myButtons[i] ) );
--			myError = 1;
--
--            for i=1,#myTrim do
--                ShowMessage("     " .. i .. CCC(myTrim[i]) );
--            end
		elseif ( myTrim[COMMAND] == "always" ) then
			if  ( #myTrim ~= 3 ) and ( #myTrim ~= 4 ) then
				ShowMessage( "ALWAYS specification requires exactly 4 elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end
			end
		elseif ( myTrim[COMMAND] == "spell" ) then
			if  ( #myTrim ~= 3 ) and ( #myTrim ~= 4 ) then
				ShowMessage( "SPELL specification requires 3 or 4 elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end
			end
		elseif ( myTrim[COMMAND] == "spellblink" ) then
			if  ( #myTrim ~= 3 ) and ( #myTrim ~= 4 ) then
				ShowMessage( "SPELLBLINK specification requires 3 or 4 elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end
			end
		elseif ( myTrim[COMMAND] == "health" ) or ( myTrim[COMMAND] == "healthblink" ) then
			if ( #myTrim ~= 5 ) then
				ShowMessage( "HEALTH or HEALTHBLINK specification requires exactly 5 elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end
            else
                local x,id = XUB_GetSpellTextureAndID(myTrim[3],myTrim[4]);
                if ( id == 0 ) then
                    ShowMessage( "Error: HEALTH or HEALTHBLINK button does not refer to a known spell."  .. CCC( myTrim[3] ) );
                    myError = 1;

                    for i=1,#myTrim do
                        ShowMessage("     " .. i .. CCC(myTrim[i]) );
                    end

                end
			end
		elseif ( myTrim[COMMAND] == "macro" ) then

			if ( #myTrim < 4 ) then
				ShowMessage( "MACRO specification requires 4 or more elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end
			elseif ( myTrim[4]:sub(1,1) ~= "/" ) then
				ShowMessage( "MACRO specification requires that you use the '/' for the macro in the specification."  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end

			end

		elseif ( myTrim[COMMAND] == "debuff" ) or ( myTrim[COMMAND] == "debuffblink" ) then
			if ( #myTrim ~= 5 ) then
				ShowMessage( "DEBUFF or DEBUFFBLINK specification requires 5 elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end

            else
                local x,id = XUB_GetSpellTextureAndID(myTrim[3],myTrim[4]);
                if ( id == 0 ) then
                    ShowMessage( "Error: DEBUFF or DEBUFFBLINK button does not refer to a known spell."  .. CCC( myTrim[3] ) );
                    myError = 1;
                    for i=1,#myTrim do
                        ShowMessage("     " .. i .. CCC(myTrim[i]) );
                    end
                end
			end
		elseif ( string.sub(myTrim[COMMAND],1,4) == "buff") then
			if ( #myTrim ~= 5 ) then
				ShowMessage( "BUFF or BUFFBLINK specification requires 5 elements"  .. CCC( myButtons[i] ) );
				myError = 1;

                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end

            else
--                ShowMessage("string.upper(myTrim[4])" .. string.upper(myTrim[4]) );

                local l_MyRank = myTrim[4]
                if string.find( string.upper(l_MyRank), "RANKED" ) then
                    l_MyRank = RANK .. " " .. 1
                end
                local x,id = XUB_GetSpellTextureAndID( myTrim[3], l_MyRank );
                if ( id == 0 ) then
                    ShowMessage( "Error: BUFF button does not refer to a known spell."  .. CCC( myTrim[3] ) );
                    myError = 1;
                    for i=1,#myTrim do
                        ShowMessage("     " .. i .. CCC(myTrim[i]) );
                    end
                end
			end
		else
			ShowMessage( "Unknown button specification - accepts: spell, spellblink, buff, buffblink, debuff, debuffblink, health, healthblink, use, always or macro." );
            for i=1,#myTrim do
			    ShowMessage("     " .. i .. CCC(myTrim[i]) );
		    end

			myError = 1;
		end
	end

	return myError;

end

function XUB_TestGroups()

	local myError = 0;

	local myButtons = XUB_SplitTextIntoTable( XUB_ConfigPanel2GroupsEdit:GetText() );

	for i = 1,# myButtons do
		local myTrim,myItems = XUB_SplitCommaAndTrim(myButtons[i]);
		if ( #myTrim ~= 3 ) and ( #myTrim ~= 4 ) then
			ShowMessage( "GROUP must have 3 or 4 items"  .. CCC( myButtons[i] ) );
			myError = 1;
        else
            myError = 1;
            if ( myTrim[3] == "*" ) or ( myTrim[3] == "party" ) or ( myTrim[3] == "partypet" ) or ( myTrim[3] == "raid" ) or ( myTrim[3] == "raidpet" ) or ( string.upper(myTrim[3]) == "MB-TARGET" ) or ( string.upper(myTrim[3]) == "KB-TARGET" ) then
                myError = 0;
            else
                for i=1,#XUB_UnitList do
                    if ( myTrim[3] == XUB_UnitList[i] ) then
                        myError = 0;
                    end
                end
            end

            if ( myError == 1 ) then
                ShowMessage( "Error: Group specification referred to an unknown unit '" .. CCC(myTrim[3]) .. "'." );
                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end

            end
		end
	end

	return myError;

end

function XUB_TestProfiles()

	local myError = 0;
	local myButtons = XUB_SplitTextIntoTable( XUB_ConfigPanel3ProfilesEdit:GetText() );

    XUB_FirstProfile = "";

	for i = 1,# myButtons do
		local myTrim,myItems = XUB_SplitCommaAndTrim(myButtons[i]);

		if ( #myTrim ~= 3 ) and ( #myTrim ~= 5 ) then
			ShowMessage( "PROFILE must have 3 or 5 items"  .. CCC( myButtons[i] ) );
			myError = 1;
        else
            myError = 1;
            if ( myTrim[2] == "float" ) or ( myTrim[2] == "party" ) or ( myTrim[2] == "partypet" ) or ( myTrim[2] == "raid" ) or ( myTrim[2] == "raidpet" ) then
                myError = 0;
            else
                for i=1,#XUB_UnitList do
                    if ( myTrim[2] == XUB_UnitList[i] ) then
                        myError = 0;
                    end
                end
            end

            if ( myError == 0 ) then
                if ( i == 1 ) then
                    -- Name of the first profile
                    XUB_FirstProfile = myTrim[1];
                end
            else
                ShowMessage( "Error: profile specification referred to an unknown unit '" .. myTrim[2] .. "'." );
                for i=1,#myTrim do
                    ShowMessage("     " .. i .. CCC(myTrim[i]) );
                end
            end

		end
	end

	return myError;

end


function XUB_Save()
	XUB_SaveButtons();
	XUB_SaveGroups();
	XUB_SaveProfiles();

    if ( XUB_CurrentProfile == "" ) and ( XUB_FirstProfile ~= "" ) then
        XUB_CurrentProfile = XUB_FirstProfile;
    end

    XUB_DelayTime = XUB_DelayTime_after_manualProfileChange;
    XUB_ApplyProfile(XUB_CurrentProfile);

    XUB_ConfigSaveAndExit:Hide();
    XUB_ConfigCloseBtnText:SetText("Close");
end

function XUB_GetMacro()
    XUB_Config:Hide();

	XUB_SelectMacroReturnScreenLabel:SetText("XUB_Config");
	XUB_SelectMacro:Show();
end

function XUB_C_MakeGroups()
    XUB_Config:Hide();

    XUB_SelectSpellReturnScreenLabel:SetText("XUB_Config");
    XUB_SelectSpell:Show();
end

function XUB_C_MakeProfiles()
    ShowMessage("Make Profiles is not implemented.  It is coming soon.");
    --XUB_Config:Hide();

    --XUB_SelectSpellReturnScreenLabel:SetText("XUB_Config");
    --XUB_SelectSpell:Show();
end



function XUB_SaveButtons()

	XUB_ConfigButtons = {};

	--local myButtons = XUB_SplitTextIntoTable( XUB_ConfigPanel0ReferenceEdit:GetText());

	--for i = 1,# myButtons do
	--	local myItems = { strsplit(",",myButtons[i]) };
	--	table.insert(XUB_ConfigButtons,myItems);
	--end

	local myButtons = XUB_SplitTextIntoTable( XUB_ConfigPanel1ButtonsEdit:GetText() );

	for i = 1,# myButtons do
		local myTrim,myItems = XUB_SplitCommaAndTrim(myButtons[i]);

		if ( myItems[2] == "macro" ) then
			local myMacroItems = {};

			-- Find the location of the "/"
			local p = myItems[4]:find("\/");
			myMacroItems[4] = myItems[4]:sub(p);

			myMacroItems[1] = strtrim(myItems[1]);
			myMacroItems[2] = strtrim(myItems[2]);
			myMacroItems[3] = strtrim(myItems[3]);

            local myCommand = "";
            local myPrefix = "";
            for i = 4,#myItems do
                myCommand = myCommand .. myPrefix .. myItems[i];
                myPrefix = ",";
            end

            myMacroItems[4] = myCommand;

			table.insert(XUB_ConfigButtons,myMacroItems);
		else
			table.insert(XUB_ConfigButtons,myTrim);
		end
	end

end

function XUB_SaveGroups()

	XUB_ConfigGroup = {}

	local myGroups = XUB_SplitTextIntoTable( XUB_ConfigPanel2GroupsEdit:GetText() );

	XUB_ConfigGroups = {};

	for i = 1,# myGroups do
		local myTrim,myItems = XUB_SplitCommaAndTrim(myGroups[i]);
		table.insert(XUB_ConfigGroups,myTrim);
	end

end

function XUB_SaveProfiles()

	local myProfiles = XUB_SplitTextIntoTable( XUB_ConfigPanel3ProfilesEdit:GetText() );

	XUB_ConfigProfiles = {};

	for i = 1,# myProfiles do
		local myTrim,myItems = XUB_SplitCommaAndTrim(myProfiles[i]);
		table.insert(XUB_ConfigProfiles,myTrim);
	end

end

function XUB_SplitTextIntoTable(aText)

	local s=1;
	local myLine = 0;
	local myLen = aText:len();
	local mySource = {};

	for i = 1,myLen do
		if ( aText:sub(i,i) == "\n" ) then
			if ( strtrim(aText:sub(s,i-1)) ~= "" ) then
				myLine = myLine + 1
				mySource[myLine] = strtrim(aText:sub(s,i-1));
			end
			s = i+1;
		end
	end

	if ( myLen > 0 ) then
		if ( strtrim(aText:sub(s,myLen)) ~= "" ) then
			myLine = myLine + 1
			mySource[myLine] = strtrim(aText:sub(s,myLen));
		end
	end

	return mySource;
end

function XUB_SplitCommaAndTrim(aText)
    local myTrim = {}
    local myItems = { strsplit(",",aText) };

    for i=1,#myItems do
        myTrim[i] = strtrim(myItems[i]);
    end
    return myTrim,myItems
end


---------------------------------------------------------------------------------------------------------------------------
-- CONFIGURE HEALS                                                                                                       --
---------------------------------------------------------------------------------------------------------------------------
function XUB_ChangeSpell()
	local junk;
	junk,XUB_SS_SelectedSpell = XUB_GetSpellTextureAndID(XUB_ConfigSpells[XUB_SelectedConfigSpell][3],XUB_ConfigSpells[XUB_SelectedConfigSpell][4])
end

