--do   -- BIG Loop to keep variables 'static'

-- Here most of the variables that the OnUpdate code uses are declared quasi 'static'
-- The reason is: before this change, all these variables were created on each OnUpdate cycle
-- and garbage collected afterwards. While this was no problem in memory consumption due to the
-- garbage collection, it used some processing power to do this.
-- So I decided to make these varibales static, increasing the memory footprint but gaining performance.
--                 tay

-- from function XUB_UpdateUnitButtons2(aUnit)
-- from function XUB_GetWeaponEnchantName(aSlot)

-- from function XUB_CHECK_BUFF_DEBUFF_STATE(bUnit,bBuffs,bDebuffs)

--from function XUB_CHECK_SINGLE_BUTTON(myButton, cBuffs, cDebuffs)

-- from function UpdateBuff(myButton,aBuffs)

-- from function UpdateHealth(myButton)

-- from function UpdateDebuff(myButton,aBuffs,aDebuffs)

-- from function UpdateSpell(myButton,aBuffs,aDebuffs)

-- from function XUB_Generate_CountdownText(myButton)


--
-- Performs a periodic update of the buttons according to XUB_UPDATERATE which defaults to 2 seconds
--
function XUB_OnUpdate(self,elapsed)

--    XUB_CheckProfiling();

    XUB_NextUpdate = XUB_NextUpdate - elapsed;

    if ( XUB_APPLY_DELAY >= 0 ) then
        XUB_APPLY_DELAY = XUB_APPLY_DELAY - elapsed;

        if ( XUB_APPLY_DELAY < 0 ) then

            if ( XUB_MakeInitialisation ) then      -- means: Create the buttons the first time after login/reload
                                                    -- for XUB, this could be done at PLAYER_LOGIN or PLAYER_ENTERING_WORLD
                XUB_MakeInitialisation = nil;       -- However, at this some other addons have not created their frames yet.
                                                    -- If we are in a party or raid, then this is a problem ;-)
                XUB_Initialize_Raidtable();         -- To circumvent this we first create buttons here in the OnUpdate routine

                if ( XUB_CurrentProfile ~= "" ) then
                    --ShowMessage("XUB: First time applying of a profile");
                    XUB_ApplyProfile(XUB_CurrentProfile);
                    --ShowMessage("XUB: First time applying of a profile done");
                else
                    ShowMessage("XUB has not been configured yet.")
                    ShowMessage("    Please make a config by entering  /XUB config");
                end
                return;
            end

            -- Checks for the following changes:
            XUB_Check_for_Changes();         -- party/raid changes | newly learned/unlearned spells | profile change (user driven)

        end

    end

    if ( XUB_MakeButtons == 1 ) then        -- This is true if the OnUpdate event fires while we are making buttons
        return;
    end

    if ( XUB_NextUpdate > 0 ) then
        return;
    end

    --Debug( "XUB_APPLY_DELAY" , XUB_APPLY_DELAY );

    XUB_NextUpdate = XUB_UPDATERATE;        -- When do we come by this way next?
    x,XUB_TIME = math.modf(XUB_TIME+XUB_BLINKRATE);

    -- if there is a button that is currently mouse overed and in the State COOLDOWN then UpdateThisButtonsTooltip
    if ( XUB_MouseOveredButton ) then
        if ( XUB_MouseOveredButton.isSTATE_COOLDOWN ) then
            XUB_UpdateTooltip(XUB_MouseOveredButton);
        end
    end

    XUB_UpdateButtons();

    XUB_HealthLeftTarget = 1000000;    -- The approximate % to damage ratio

end

--
-- Updates buttons on the screen according to unit.  To a maximum of the number
-- of units on the screen or the XUB_QUEUESIZE (which ever is smaller).
--
function XUB_UpdateButtons()

    local MaxToUpdate = math.min( #XUB_UPDATEUNITS,XUB_QUEUESIZE );

    for i = 1,MaxToUpdate do

        if ( XUB_QUEUEPTR > #XUB_UPDATEUNITS ) then
            XUB_QUEUEPTR = 1;
        end

        XUB_UpdateUnitButtons2(XUB_UPDATEUNITS[XUB_QUEUEPTR]);
        XUB_QUEUEPTR = XUB_QUEUEPTR + 1;
    end
end


-- This function updates all of the buttons that use the specified aUnit as destination unit
function XUB_UpdateUnitButtons2(aUnit)

    local Buffs = ",";
    local Debuffs = ",";

    if ( aUnit ~= "float" ) then
      -- Check all Buffs and Debuffs of the current Unit
      Buffs, Debuffs = XUB_CHECK_BUFF_DEBUFF_STATE(aUnit,Buffs,Debuffs);
    end

    local myButton
    local myExitByNil = 1;            -- This vairable is never changed, the routine is only left by 'return'
    local buttonCounter = 1;          -- Main counter, used for cycling through all buttons for this unit

    while ( myExitByNil == 1 ) do

        --Get a SINGLE Button to work with it
        myButton = getglobal( XUB_GenerateButtonName(aUnit,buttonCounter) );
        --ShowMessage( "TESTING '" .. XUB_GenerateButtonName(aUnit,buttonCounter) .. "'");

        -- If we encounter a nil, it means this button sequence is nil and we don't need
        -- to test any more buttons on this unit.
        if ( myButton == nil ) then
            return;
        -- A button's isActive is set to 0 if it is hidden after a profile change
        elseif ( myButton.isActive == 0 ) then
            --ShowMessage("Been here with inactive buttons");
            return;
        end

        -- Next line can be enabled if you really need to see which units/buttons get updated. This may generate a lot of spam. Be warned.
--        if ( XUB_DEBUG > 0 ) then XUB_Show_DebugBlock1(myButton, buttonCounter, aUnit);           end

        local myPreviousState = myButton.State;
        local wasSTATE_INVALID = myButton.isSTATE_INVALID;

        XUB_CHECK_SINGLE_BUTTON(myButton, Buffs, Debuffs)

        myButton.State =  0
        --if myButton.isSTATE_ENABLED    then myButton.State = myButton.State + 0; end
        if myButton.isSTATE_HEALTHOFF  then myButton.State = myButton.State + 1; end
        if myButton.isSTATE_BUFFED     then myButton.State = myButton.State + 2; end
        if myButton.isSTATE_DIMMED     then myButton.State = myButton.State + 4; end
        if myButton.isSTATE_COOLDOWN   then myButton.State = myButton.State + 8; end
        if myButton.isSTATE_NO_ITEMS   then myButton.State = myButton.State + 16; end
        if myButton.isSTATE_OUTOFMANA  then myButton.State = myButton.State + 32; end
        if myButton.isSTATE_OUTOFRANGE then myButton.State = myButton.State + 64; end
        if myButton.isSTATE_INVALID    then myButton.State = myButton.State + 128; end

        -- Blinking must be handled no matter if the button state did not change or not
        if ( myButton.Blink == 1 ) and ( myButton.State < 1 ) then --if the button should blink whenever he is fully enabled
            XUB_HandleBlinking(myButton);
        elseif ( myButton.BlinkLastSeconds ) then       -- if this Button is intended to blink in the last seconds of a buff
            if ( myButton.RemainingBufftime ) then  -- if this Button has a remaining buff time
                if ( tonumber(myButton.RemainingBufftime) < tonumber(myButton.BlinkLastSeconds) ) then
                    XUB_HandleBlinking(myButton);
                end
            end
        end

        -- Countdown for buttons in cooldown must be handled no matter if the button state did change or not
        if ( myButton.isSTATE_COOLDOWN ) then
            XUB_Set_CountdownText(myButton);
        end

        if myButton.isSTATE_INVALID and wasSTATE_INVALID then
            -- Do nothing, the button was hidden and is still hidden
        else
            if ( myButton.State ~= myPreviousState ) then

                if ( XUB_DEBUG > 0 ) then   XUB_Show_DebugBlock2(myButton, buttonCounter, aUnit, myPreviousState );          end

                --now then lets do some real action with the current button
                XUB_CHANGE_SINGLE_BUTTON(myButton);

            end
        end

        if ( XUB_DEBUG > 0 ) and ( myButton.isSTATE_BUFFED ) and ( XUB_MouseOveredButton ) and ( XUB_MouseOveredButton == myButton ) then  XUB_UpdateTooltip(XUB_MouseOveredButton);   end

        buttonCounter = buttonCounter + 1;
    end

end


function XUB_GetWeaponEnchantName(aSlot)


    XUB_WeaponTooltip:ClearLines();
    XUB_WeaponTooltip:SetInventoryItem("player", aSlot);

    local WeaponBuffText, WeaponBuffName, brack;

    for i=1, XUB_WeaponTooltip:NumLines(), 1 do
        WeaponBuffText = getglobal("XUB_ParseToolTipTextLeft"..i):GetText();
        WeaponBuffName = strmatch(WeaponBuffText, "^.+ %(%d+%s+.+%)$");
        if ( WeaponBuffName ) then
            break;
        end
    end

    if ( WeaponBuffName ) then
        brack = WeaponBuffName:find(" %(");
        if ( brack == nil ) then
            return WeaponBuffName;
        else
            return WeaponBuffName:sub(1,brack-1);
        end
    else
        return "nil";
    end
end

function XUB_CHECK_BUFF_DEBUFF_STATE(bUnit,bBuffs,bDebuffs)

    local name, rank, texture, count, duration, expiryTime, debuffType, untilCanceled;
    local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges;
    local castable, buffCounter, removable

    if ( bUnit == "player" ) then
        hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo();
        if ( mainHandExpiration ~= nil ) then
            bBuffs = bBuffs .. XUB_GetWeaponEnchantName(16) .. ",";
        end
        if ( offHandExpiration ~= nil ) then
            bBuffs = bBuffs .. XUB_GetWeaponEnchantName(17) .. ",";
        end
    end

    -- name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitAura("unit", [index] or ["name", "rank"][, "filter"])
    -- Filter: HELPFUL HARMFUL PLAYER RAID CANCELABLE NOT_CANCELABLE

    -- Get the Buffs on the Unit
    buffCounter=1;
    name, rank, texture, count, debuffType, duration, expiryTime, isMine, isStealable = UnitAura(bUnit, buffCounter, "HELPFUL");
    while ( name ~= nil ) do
        if buffcounter == 1 then
            ShowMessage( "name: " .. CCC(name));
            ShowMessage( "rank: " .. CCC(rank));
            ShowMessage( "count: " .. CCC(count));
            ShowMessage( "debuffType: " .. CCC(debuffType));
            ShowMessage( "duration: " .. CCC(duration));
            ShowMessage( "expiryTime: " .. CCC(expiryTime));
            ShowMessage( "untilCanceled: " .. CCC(untilCanceled));
        end

        if ( expiryTime ) then
            name = name .. expiryTime;
        end
        bBuffs = bBuffs .. name .. ","

        buffCounter=buffCounter+1;
        name, rank, texture, count, debuffType, duration, expiryTime, isMine, isStealable = UnitAura(bUnit, buffCounter, "HELPFUL");
    end

    -- Get the Debuffs on the Unit
    buffCounter=1;
    name, rank, texture, count, debuffType, duration, expiryTime, isMine, isStealable = UnitAura(bUnit, buffCounter, "HARMFUL");

--[[
        ShowMessage( "name: " .. CCC(name));
        ShowMessage( "rank: " .. CCC(rank));
        ShowMessage( "count: " .. CCC(count));
        ShowMessage( "debuffType: " .. CCC(debuffType));
        ShowMessage( "duration: " .. CCC(duration));
        ShowMessage( "timeLeft: " .. CCC(timeLeft));
--]]

    while ( name ~= nil ) do

        bBuffs = bBuffs .. name .. ",";

        if ( debuffType ~= nil ) then
            bDebuffs = bDebuffs .. debuffType .. ",";
        else
            bDebuffs = bDebuffs .. name .. ",";
        end

        buffCounter=buffCounter+1
        name, rank, texture, count, debuffType, duration, expiryTime, isMine, isStealable = UnitAura(bUnit, buffCounter, "HARMFUL");
    end

  return bBuffs, bDebuffs;
end


function XUB_CHECK_for_GLOBAL_DIM(myButton)

    if ( XUB_USE_GLOBALDIM == 1 ) then                      -- if Global Dim is enabled

        if ( XUB_GLOBALDIM_active == 1 ) then               -- if Global Dim is active this moment
            myButton.isSTATE_DIMMED = 1;
        end

    end
end


function XUB_CHECK_SINGLE_BUTTON(myButton, cBuffs, cDebuffs)

    local myPreviousCooldown = myButton.MyCooldown;

    -- We start assuming everything is OK.
--    myButton.isSTATE_ENABLED    = 1;
    myButton.isSTATE_HEALTHOFF  = nil;
    myButton.isSTATE_BUFFED     = nil;
    myButton.isSTATE_DIMMED     = nil;
    myButton.isSTATE_COOLDOWN   = nil;
    myButton.isSTATE_NO_ITEMS   = nil;
    myButton.isSTATE_OUTOFMANA  = nil;
    myButton.isSTATE_OUTOFRANGE = nil;
    myButton.isSTATE_INVALID    = nil;
    myButton.MyCooldown = 0;

--    ShowMessage("UnitCheck: "..CCC(UnitName(myButton.DestinationUnit)));
    if ( myButton.Bar == nil ) then                                 -- does the button have a 'unit' where it is attached to? (FLOAT counts as unit)
        myButton.isSTATE_INVALID = 1;
        return;
    elseif ( UnitExists(myButton.DestinationUnit) == nil ) then     -- does the destination unit exist ?
        myButton.isSTATE_INVALID = 1;
        return;
    elseif ( UnitName(myButton.DestinationUnit) == nil ) then       -- do we get a name for the destination unit ?
        myButton.isSTATE_INVALID = 1;
        return;
    elseif ( myButton.Bar ~= "float" ) then             -- do not perform the next checks for the float bar, since they can never work
        if ( UnitExists(myButton.Bar) == nil ) then                 -- does the unit where the button is attached to exist ?
            myButton.isSTATE_INVALID = 1;
            return;
        elseif ( UnitName(myButton.Bar) == nil ) then               -- do we get a name of the unit where the button is attached to
            myButton.isSTATE_INVALID = 1;
            return;
        end
    end

    if ( UnitIsDeadOrGhost("player") ) then
        myButton.isSTATE_INVALID = 1;
        return;
    end

    if ( UnitIsDeadOrGhost(myButton.DestinationUnit) ) then
        if myButton.Debuff == "Death" then
--            myButton.isSTATE_ENABLED = 1;
            -- no return, because we need to continue. Unit could still be out of range or out of mana etc.
        else
            myButton.isSTATE_INVALID = 1;
            return;
        end
    end

    if ( myButton.isMacro == 1 ) then
        -- Do Nothing -- a macro is always enabled
    elseif ( myButton.isUse == 1 ) then

        if ( myButton.NoIcon == 1 ) then

            myButton.NoIcon = 0;

            local USEitemTexture
            _, _, _, _, _, _, _, _, _, USEitemTexture = GetItemInfo(myButton.UseItemName)

            if ( USEitemTexture == nil ) then
                USEitemTexture = "Interface\Icons\INV_Misc_QuestionMark";
                myButton.NoIcon = 1;
            elseif ( USEitemTexture == "" ) then
                USEitemTexture = "Interface\Icons\INV_Misc_QuestionMark";
                myButton.NoIcon = 1;
            end

            if ( myButton.NoIcon == 0 ) then
                myButton:SetNormalTexture(USEitemTexture)
                myButton:SetHighlightTexture(USEitemTexture)
            end

        end

        -- Check if the target is in Range ?
        if ( myButton.DestinationUnit ~= "player" ) then                           -- no check needed for player, since 'player' is always in range ;-)))
            local inRange = IsItemInRange( myButton.UseItemName , myButton.DestinationUnit );
            if ( inRange == nil ) then
                myButton.isSTATE_INVALID = 1;
                return;
            elseif ( inRange == 0 ) then
                myButton.isSTATE_OUTOFRANGE = 1;
            end
        end

        -- Check if the item is usable at the moment ?
        -- myUsable, myNoMana = IsUsableItem( myButton.UseItemName );  -- this is supposed to be valid since 2.1, but ...
        local myUsable = IsUsableItem( myButton.UseItemName );
        if ( myUsable == nil ) then
            myButton.isSTATE_NO_ITEMS = 1;
            return;
        end

        -- Check if the item is on Cooldown
        local start, duration, enable = GetItemCooldown(myButton.UseItemName);
        if enable > 0 then
            local remaining = (duration - (GetTime() - start));
            if ( duration > 0 ) and ( remaining > 0 ) then
                if remaining > myPreviousCooldown then                  -- this fixes the fact that some items only report the global CD first
                    --ShowMessage("remaining: '" .. remaining .. "'   old: '" .. myPreviousCooldown .. "'" )
                    if XUB_USE_BUILTIN_COOLDOWN == 1 then
                        CooldownFrame_SetTimer(myButton.CooldownFrame, GetTime(), remaining, 1);
                    end
                end
                myButton.isSTATE_COOLDOWN = 1;
                myButton.MyCooldown = remaining;

                if ( XUB_ShowGlobalCooldown ) or (( duration ~= 1.5 ) and ( duration ~= 1.0 )) then
                    myButton.ShowCountdown = 1;
                end
                return;
            end
        end

        XUB_CHECK_for_GLOBAL_DIM(myButton);

        -- Check if a buff condition is given for a certain item
        if ( myButton.UseHideItems ~= "" ) then
            local myItems = { };
            myItems = { strsplit("/",myButton.UseHideItems ) };

            local myCondition
            for di=1,#myItems do
                myCondition = myItems[di];
                if string.sub(myCondition,1,1) ~= "!" then
                    myCondition = "," .. myCondition .. ",";
                    if ( string.find(cBuffs, myCondition) ~= nil ) then
                        myButton.isSTATE_BUFFED = 1;
                        return;
                    end
                else
                    myCondition = "," .. string.sub(myCondition,2) .. ",";
                    myButton.isSTATE_BUFFED = 1; -- drome 1.9rc8 assumed buffed to hide - then unhide if debuff exists
                    if ( string.find(cDebuffs, myCondition) ~= nil ) then
-- drome removed comment on this line for 1.9rc8 testing
                        myButton.isSTATE_BUFFED = nil;
                        return;
                    end
                end
            end
        end

    elseif ( myButton.SpellID > 0 ) then

        if ( myButton.ExcludeUnits ~= "" ) then
            -- Are there any Exclude Units declared for the button?
            if ( UnitClass(myButton.DestinationUnit) ~= nil ) then
                if ( string.find( myButton.ExcludeUnits .. "/" , UnitClass(myButton.DestinationUnit) .. "/" ) ~= nil ) then
                    myButton.isSTATE_INVALID = 1;
                    return;
                end
            elseif ( UnitCreatureType(myButton.DestinationUnit) ~= nil ) then
                if ( string.find( myButton.ExcludeUnits .. "/" , UnitCreatureType(myButton.DestinationUnit) .. "/" ) ~= nil ) then
                    myButton.isSTATE_INVALID = 1;
                    return;
                end
            end
        end

        if ( myButton.isBuff == 1 ) then
            UpdateBuff(myButton,cBuffs);
        elseif ( myButton.isHeal == 1 ) then
            UpdateHealth(myButton);
        elseif ( myButton.isDebuff == 1 ) then
            UpdateDebuff(myButton,cBuffs,cDebuffs);
        else
            UpdateSpell(myButton,cBuffs,cDebuffs);
        end
 
        if (myButton.isSTATE_INVALID) or (myButton.isSTATE_COOLDOWN) then  return;   end

        XUB_CHECK_for_GLOBAL_DIM(myButton);

        if myButton.isSTATE_INVALID then  return;   end

       -- Disable the button if it is not in range or valid for the unit or if that unit is in a duel
        if ( myButton.DestinationUnit ~= "player" ) then
            local inRange = IsSpellInRange(myButton.FullSpellName,myButton.DestinationUnit)
            if ( inRange == nil ) then
                -- this means either (from WoWWiki)
                -- 1. The spell cannot be cast on the unit. i.e. attempting to check range using 'Frostbolt' on a party member will always return 'nil', similarly testing 'Heal' against an enemy target will also return nil.
                -- 2. If the unit is not 'visible' (see API UnitIsVisible) then you will get a nil, and not a '0' as you might expect.
                -- 3. The unit doesnt exist (e.g. 'target' when you have nothing targetted)
                -- 4. The current player does not know this spell (so you cannot use 'Heal' to test 40 yard range for anyone other than a priest)
                local really_OutOfRange = 1;
                --TAYEDAEN's QUICKFIX for Blizzard bug:
                for i = 1, #XUB_IsSpellInRange_Exceptions do
                    if string.find(myButton.FullSpellName, XUB_IsSpellInRange_Exceptions[i]) then
                        --Debug( CCC(XUB_IsSpellInRange_Exceptions[i]) .. " is always enabled because he is included in XUB_IsSpellInRange_Exceptions.lua"  );
                        really_OutOfRange = nil;
                    end
                end
                --Spell cannot be casted (but not because of Mana reasons)
                if really_OutOfRange then
                    myButton.isSTATE_INVALID = 1;
                end
            elseif ( inRange == 0 ) then
                -- this means the target is not in Range
                myButton.isSTATE_OUTOFRANGE = 1;
            end
        end

        -- check if button is in cooldown (or if duration=1.5/1.0 then it's global cooldown)
        local start, duration, enable = GetSpellCooldown(myButton.SpellID, SpellBookFrame.bookType);
        if enable > 0 then
            local remaining = (duration - (GetTime() - start));
            if ( duration > 0 ) and ( remaining > 0 ) then
                if remaining > myPreviousCooldown then                  -- this fixes the fact that some spells only report the global CD first
                    --ShowMessage("remaining: '" .. remaining .. "'   old: '" .. myPreviousCooldown .. "'" )
                    if XUB_USE_BUILTIN_COOLDOWN == 1 then
                        CooldownFrame_SetTimer(myButton.CooldownFrame, GetTime(), remaining, 1);
                    end
                end
                myButton.isSTATE_COOLDOWN = 1;
                myButton.MyCooldown = remaining;

                if ( XUB_ShowGlobalCooldown ) or (( duration ~= 1.5 ) and ( duration ~= 1.0 )) then
                    myButton.ShowCountdown = 1;
                end
                return;
            end
        end

        -- Does the player have sufficient Mana/Rage/Energy and skill to cast the spell?
        local myUsable, myNoMana = IsUsableSpell(myButton.FullSpellName);
        if (not myUsable) then
            --Spell cannot be casted
            if (not myNoMana) then
                local really_notusable = 1;
                --TAYEDAEN's QUICKFIX for Blizzard bug:
                for i = 1, #XUB_IsUsableSpell_Exceptions do
                    if string.find(myButton.FullSpellName, XUB_IsUsableSpell_Exceptions[i]) then
                        --Debug( CCC(XUB_IsUsableSpell_Exceptions[i]) .. " is always enabled because he is included in XUB_IsUsableSpell_Exceptions.lua"  );
                        really_notusable = nil;
                    end
                end
                --Spell cannot be casted (but not because of Mana reasons)
                if really_notusable then
                    myButton.isSTATE_INVALID = 1;
                end
            else
                --not enough mana to cast the spell
                myButton.isSTATE_OUTOFMANA = 1;
            end
        end


    end

end


function XUB_CHANGE_SINGLE_BUTTON(myButton)

    -- Set the defaults for all buttons
    myButton:GetNormalTexture():SetVertexColor(1,1,1);
    myButton:GetHighlightTexture():SetAlpha( 0 );  --make the button look like its unusable if mouse overed

    if myButton.isSTATE_INVALID then
        myButton:SetAlpha( XUB_INVALID_ALPHA );
        --ShowMessage("myButton.FullSpellName: " .. CCC(myButton.FullSpellName));
    elseif myButton.isSTATE_OUTOFRANGE then
        if ( myButton.isSTATE_BUFFED ) and ( XUB_HEALACTION == "Show as Disabled" ) then
                    myButton:SetAlpha( XUB_INVALID_ALPHA );
        elseif ( XUB_COLORUSAGE ~= "Transparency" ) then
            myButton:GetNormalTexture():SetVertexColor(XUB_COLORS["c_range"].r, XUB_COLORS["c_range"].g,XUB_COLORS["c_range"].b);
            myButton:SetAlpha( XUB_ENABLE_ALPHA * 0.7 );
        else
            myButton:SetAlpha( XUB_OUTOFRANGE_ALPHA );
        end
    elseif myButton.isSTATE_OUTOFMANA then
        if ( myButton.isSTATE_BUFFED ) and ( XUB_HEALACTION == "Show as Disabled" ) then
                    myButton:SetAlpha( XUB_INVALID_ALPHA );
        elseif ( XUB_COLORUSAGE ~= "Transparency" ) then
            myButton:GetNormalTexture():SetVertexColor(XUB_COLORS["c_outofmana"].r, XUB_COLORS["c_outofmana"].g,XUB_COLORS["c_outofmana"].b);
            myButton:SetAlpha( XUB_ENABLE_ALPHA * 0.9 );
        else
            myButton:SetAlpha( XUB_NOMANA_ALPHA );
        end
    elseif myButton.isSTATE_NO_ITEMS  then
        myButton:SetAlpha( XUB_NOMANA_ALPHA );
    else    -- the rest of the states is NON-exclusive !!

        myButton:GetNormalTexture():SetVertexColor(1,1,1);
        myButton:SetAlpha( XUB_ENABLE_ALPHA );

--        if myButton.isSTATE_ENABLED   then
--            myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
--        end

        if myButton.isSTATE_DIMMED    then
            if ( XUB_COLORUSAGE ~= "Transparency" ) then
                myButton:GetNormalTexture():SetVertexColor(XUB_COLORS["c_globaldim"].r, XUB_COLORS["c_globaldim"].g,XUB_COLORS["c_globaldim"].b);
                myButton:SetAlpha( XUB_ENABLE_ALPHA );
            else
                myButton:SetAlpha( XUB_GLOBLADIM_ALPHA );
            end
        end

        if myButton.isSTATE_BUFFED    then
            if ( XUB_HEALACTION == "Show as Disabled" ) then
                if myButton.isShowAlways then
                    myButton:SetAlpha( XUB_COOLDOWN_ALPHA );
                else
                    myButton:SetAlpha( XUB_INVALID_ALPHA );
                end
            else
                myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
                if ( XUB_COLORUSAGE ~= "Transparency" ) then
                    myButton:GetNormalTexture():SetVertexColor(XUB_COLORS["c_buffed"].r, XUB_COLORS["c_buffed"].g,XUB_COLORS["c_buffed"].b);
                    myButton:SetAlpha( XUB_ENABLE_ALPHA * 0.7 );
                else
                    myButton:SetAlpha( XUB_BUFFED_ALPHA );
                end
            end
        end

        if myButton.isSTATE_HEALTHOFF then
            if ( XUB_HEALACTION == "Show as Disabled" ) then
                if myButton.isShowAlways then
                    myButton:SetAlpha( XUB_COOLDOWN_ALPHA );
                else
                    myButton:SetAlpha( XUB_INVALID_ALPHA );
                end
            else  -- if XUB_HEALACTION == "Show as Buffed" then
                myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
                myButton:SetAlpha( XUB_BUFFED_ALPHA );
            end
        end

        if myButton.isSTATE_COOLDOWN  then
    --        if ( tonumber(myButton.UsefulState) <  tonumber(STATE_COOLDOWN) ) then        --do show cooldown only for states where the button could be pressed otherwise
                if ( XUB_COLORUSAGE ~= "Transparency" ) then
                    myButton:GetNormalTexture():SetVertexColor(XUB_COLORS["c_cooldown"].r, XUB_COLORS["c_cooldown"].g,XUB_COLORS["c_cooldown"].b);
                    myButton:SetAlpha( myButton:GetAlpha() * 0.8 );
                else
                    myButton:SetAlpha( XUB_COOLDOWN_ALPHA );
                end
    --        end
            if ( XUB_USE_BUILTIN_COOLDOWN == 1 ) then
                CooldownFrame_SetTimer(myButton.CooldownFrame, GetTime() , myButton.MyCooldown, 1);
            end
            XUB_Set_CountdownText(myButton);
        end


    end
end

function UpdateBuff(myButton,dBuffs)


    local buttonBuffs = { strsplit("/",myButton.Buff .. "/" .. myButton.Spell ) };
    --buttonBuffs = { strsplit("/",myButton.Buff ) };

    --ShowMessage("myButton.Buff: " .. myButton.Buff);

    local myStartPosition = nil
    local myCommaPosition = nil
    local myEndPosition, timeleft

    for di=1,#buttonBuffs do
        myStartPosition, myEndPosition = string.find(dBuffs,buttonBuffs[di]);
        if ( myStartPosition ) then
            myButton.isSTATE_BUFFED = 1;

            --if this button is intended to blink in its last seconds
            if ( myButton.BlinkLastSeconds ) then

                -- extract the endtime from the found buff (if there is an endtime attached)
                myCommaPosition = string.find(dBuffs, ",", myEndPosition);
                if myCommaPosition ~= myEndPosition+1 then
                    timeLeft = string.sub(dBuffs, myEndPosition+1, myCommaPosition - 1);
                    if ( tonumber(timeLeft) ) then
                        myButton.RemainingBufftime = timeLeft;
                    end
                end
            end

            break;
        end
    end

end


function UpdateHealth(myButton)

    if ( myButton.Threshold > 0 ) then

        local myUnitDamage = UnitHealthMax(myButton.DestinationUnit) - UnitHealth(myButton.DestinationUnit);

        if ( myUnitDamage < myButton.Threshold ) then
            myButton.isSTATE_HEALTHOFF = 1;
        end

    elseif ( myButton.Threshold < 0 ) then

        local myShow = 0;

        if ( myButton.DestinationUnit ~= "target" ) then
            myButton.isSTATE_INVALID = 1;
        else
            if ( XUB_LastHealthTarget > 0 ) then
                local healthDiff = XUB_LastHealthTarget - UnitHealth("target");
                if ( healthDiff > 0 ) then
                    XUB_HealthLeftTarget = ( XUB_LastDamageTarget / healthDiff ) * UnitHealth("target");
                    if ( XUB_HealthLeftTarget > (-myButton.Threshold) ) then
                        myButton.isSTATE_HEALTHOFF = 1;
                    end
                else
                    myButton.isSTATE_HEALTHOFF = 1;
                end
            else
                myButton.isSTATE_HEALTHOFF = 1;
            end
        end
    else
        myButton.isSTATE_INVALID = 1;
    end


end


function UpdateDebuff(myButton,eBuffs,eDebuffs)

    --ShowMessage( "Debuffs on Unit '" .. myButton.Unit .. "' are '" .. eBuffs .. "' -- Button handles " .. myButton.Debuff );

    myButton.isSTATE_INVALID = 1;

    if myButton.Debuff == "Purge" then
        if string.len(eBuffs) > 1 then
            myButton.isSTATE_INVALID = nil;
        end
        return;
    end

    if myButton.Debuff == "Death" then
        if UnitIsDeadOrGhost(myButton.DestinationUnit) then
            myButton.isSTATE_INVALID = nil;
        end
        return;
    end

    local buttonDebuffs = { strsplit("/",myButton.Debuff ) };
    local mySearch

    for di=1,#buttonDebuffs do
--        mySearch = "," .. buttonDebuffs[di] .. ",";
        mySearch = "," .. buttonDebuffs[di];
--    ShowMessage("eDebuffs: " .. eDebuffs);
        if ( string.find(eDebuffs,mySearch) ~= nil ) then
            myButton.isSTATE_INVALID = nil;
            return;
        elseif ( string.find(eBuffs,mySearch) ~= nil ) then
            myButton.isSTATE_INVALID = nil;
            return;
        end
    end

end

function UpdateSpell(myButton,fBuffs,fDebuffs)


--    myButtonSpell = "," .. myButton.Spell .. ",";
    local myButtonSpell = "," .. myButton.Spell;

    if ( string.find(fBuffs,myButtonSpell) ~= nil ) then
        myButton.isSTATE_BUFFED = 1;
    end

end


function XUB_HandleBlinking(myButton)
    --myButton:SetAlpha( math.abs(( XUB_TIME - 0.5 ) * 2) );

    myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
    if ( XUB_TIME >= 0.5 ) then
        myButton:GetHighlightTexture():SetAlpha( 1 );       --makes the button react like it can be clicked ;-)
        myButton:GetNormalTexture():SetVertexColor(1,1,1);
        myButton:SetAlpha( XUB_ENABLE_ALPHA );
    else
        if ( XUB_COLORUSAGE ~= "Transparency" ) then
            myButton:GetNormalTexture():SetVertexColor(XUB_COLORS["c_buffed"].r, XUB_COLORS["c_buffed"].g,XUB_COLORS["c_buffed"].b);
        end
        myButton:SetAlpha( 0.2 );
    end
end

function XUB_Set_CountdownText(myButton)

    if ( XUB_COUNTDOWN == 1 ) then
        if ( myButton.ShowCountdown ~= 0 ) then
            if ( myButton.MyCooldown <= 5 ) then
                myButton.CooldownFrame.text:SetTextColor(1, 0, 0, 1);   -- red
            else
                myButton.CooldownFrame.text:SetTextColor(1, 1, 1, 1);   -- white
            end
            myButton.CooldownFrame.text:SetText(XUB_Generate_CountdownText(myButton));
        end
    end
end

function XUB_Generate_CountdownText(myButton)

    local myReadableCooldown = "";

    if ( myButton.MyCooldown > 3599 ) then
        myReadableCooldown = "" ;
    elseif ( myButton.MyCooldown > 60 ) then
        myReadableCooldown = string.format("%d",myButton.MyCooldown/60) .. "m";
    elseif ( myButton.MyCooldown > 9.9 ) then
        myReadableCooldown = string.format("%d",math.ceil(myButton.MyCooldown));
    elseif ( myButton.MyCooldown > 0.01 ) then
        if XUB_ShowDecimals then
            myReadableCooldown = string.format("%.1f", myButton.MyCooldown)
        else
            myReadableCooldown = string.format("%d",math.ceil(myButton.MyCooldown));
        end
    else
        myReadableCooldown = "";
        myButton.CooldownFrame.text:SetTextColor(1, 1, 1, 1);
    end

    return myReadableCooldown;

end


--end -- BIG Loop to keep variables 'static'